#include <stdio.h>
#include <string.h>
#include "simpletools.h"
#include "config.h"
#include "debug.h"
#include "gcode_struct.h"

static char fileline[100];

static void create_gcode_struct()
{
	uint8_t gcode_char;
	uint8_t gcode_struct_element;
	uint8_t fileline_length;
	uint8_t value_counter;

	uint32_t counter;

	// Clear any lingering data
	memset(current_gcode.G, 0, strlen(current_gcode.G));
	memset(current_gcode.M, 0, strlen(current_gcode.M));
	memset(current_gcode.T, 0, strlen(current_gcode.T));
	memset(current_gcode.S, 0, strlen(current_gcode.S));
	memset(current_gcode.P, 0, strlen(current_gcode.P));
	memset(current_gcode.X, 0, strlen(current_gcode.X));
	memset(current_gcode.Y, 0, strlen(current_gcode.Y));
	memset(current_gcode.Z, 0, strlen(current_gcode.Z));
	memset(current_gcode.E, 0, strlen(current_gcode.E));
	memset(current_gcode.F, 0, strlen(current_gcode.F));
	memset(current_gcode.Valid, 0, strlen(current_gcode.Valid));

	// Get the length of the fileline that is being parsed
	fileline_length = (uint8_t)strlen(fileline);

	// Fill current_gcode.Valid with literal zeros
	for(counter = 0; counter < GCODE_ELEMENTS; counter++)
	{
		current_gcode.Valid[counter] = '0';
	}

	value_counter = 0;

	//Parse file line into seperate field values.  Various fields will remain empty
	for(counter = 0; counter < fileline_length; counter++)
	{
		gcode_char = fileline[counter];

		// Exclude these characters from becoming part of are field entries
		if(gcode_char == 13 /*Carriage Return*/ || gcode_char == 10 /*Line Feed*/)
		{
			// Break free from this loop
			break;
		}

		else if(gcode_char == ' ')
		{
			value_counter = 0;

			// Skip the remaining iterations, because
			// we found a match, however, stay in the loop,
			// because we are not done yet
			continue;
		}

		else if(gcode_char == 'G')
		{
			// Set current field element
			gcode_struct_element = 0;
			current_gcode.Valid[0] = '1';

			continue;
		}
		else if(gcode_char == 'M')
		{
			// Set current field element
			gcode_struct_element = 1;
			current_gcode.Valid[1] = '1';

			continue;
		}
		else if(gcode_char == 'T')
		{
			// Set current field element
			gcode_struct_element = 2;
			current_gcode.Valid[2] = '1';

			continue;
		}
		else if(gcode_char == 'S')
		{
			// Set current field element
			gcode_struct_element = 3;
			current_gcode.Valid[3] = '1';

			continue;
		}
		else if(gcode_char == 'P')
		{
			// Set current field element
			gcode_struct_element = 4;
			current_gcode.Valid[4] = '1';

			continue;
		}
		else if(gcode_char == 'X')
		{
			// Set current field element
			gcode_struct_element = 5;
			current_gcode.Valid[5] = '1';

			continue;
		}
		else if(gcode_char == 'Y')
		{
			// Set current field element
			gcode_struct_element = 6;
			current_gcode.Valid[6] = '1';

			continue;
		}
		else if(gcode_char == 'Z')
		{
			// Set current field element
			gcode_struct_element = 7;
			current_gcode.Valid[7] = '1';

			continue;
		}		
		else if(gcode_char == 'E')
		{
			// Set current field element
			gcode_struct_element = 8;
			current_gcode.Valid[8] = '1';

			continue;
		}
		else if(gcode_char == 'F')
		{
			// Set current field element
			gcode_struct_element = 9;
			current_gcode.Valid[9] = '1';

			continue;
		}
		else
		{
			if(gcode_struct_element == 0)
			{
				// G Field element character
				current_gcode.G[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 1)
			{
				// M Field element character
				current_gcode.M[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 2)
			{
				// T Field element character
				current_gcode.T[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 3)
			{
				// S Field element character
				current_gcode.S[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 4)
			{
				// P Field element character
				current_gcode.P[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 5)
			{
				// X Field element character
				current_gcode.X[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 6)
			{
				// Y Field element character
				current_gcode.Y[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 7)
			{
				// Z Field element character
				current_gcode.Z[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 8)
			{
				// E Field element character
				current_gcode.E[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 9)
			{
				// F Field element character
				current_gcode.F[value_counter] = gcode_char;
				value_counter++;

				continue;
			}			
		}
	}
}

int main(void)
{
	uint32_t line_counter = 0;

	// Set debug level
	debug_flags |= DEBUG_ERROR;

	// Verify that SD card pins are defined
	#if defined SD_DO && defined SD_CLK && defined SD_DI && defined SD_CS

	// Mount SD card
	sd_mount(SD_DO, SD_CLK, SD_DI, SD_CS);

	FILE * pFile;

	// Open up a file on the SD card
	pFile = fopen("GCODE.TXT","r");

	if(pFile == NULL)
	{
		if(DEBUG && (debug_flags & DEBUG_ERROR))
		{
			print("ERROR - The file could not be opened!");
		}
	}
	else
	{
		// Recursively grab filelines
		while(fgets(fileline, 100, pFile))
		{
			// Count the filelines as we go along
			line_counter++;

			// Exclude lines that begin with certain characters
			// E.G. Remove newlines, carriage returns, and lines
			// that begin with a semi-colon.
			if(fileline[0] != 59 && fileline[0] != 13 && fileline[0] != 10)
			{
				// Fill the GCODE_STRUCT with the data
				// obtained from the fileline
				create_gcode_struct();

				// Trim off newline character from the fileline
				// returned by fgets
				fileline[strcspn(fileline, "\n")] = 0;

    				print("Fileline: %s\n", fileline);

    				print("Valid GCODE Fields: %s\n", current_gcode.Valid);

				// Check the various GCODE_STRUCT field lengths
				// to see if they contain data, and if so, then
				// print the data
				if((uint8_t)strlen(current_gcode.G) != 0)
				{
					print("G: %s, ", current_gcode.G);
				}

				if((uint8_t)strlen(current_gcode.M) != 0)
				{
					print("M: %s, ", current_gcode.M);
				}

				if((uint8_t)strlen(current_gcode.T) != 0)
				{
					print("T: %s, ", current_gcode.T);
				}

				if((uint8_t)strlen(current_gcode.S) != 0)
				{
					print("S: %s, ", current_gcode.S);
				}

				if((uint8_t)strlen(current_gcode.P) != 0)
				{
					print("P: %s, ", current_gcode.P);
				}

				if((uint8_t)strlen(current_gcode.X) != 0)
				{
					print("X: %s, ", current_gcode.X);
				}

				if((uint8_t)strlen(current_gcode.Y) != 0)
				{
					print("Y: %s, ", current_gcode.Y);
				}

				if((uint8_t)strlen(current_gcode.Z) != 0)
				{
					print("Z: %s, ", current_gcode.Z);
				}

				if((uint8_t)strlen(current_gcode.E) != 0)
				{
					print("E: %s, ", current_gcode.E);
				}

				if((uint8_t)strlen(current_gcode.F) != 0)
				{
					print("F: %s, ", current_gcode.F);
				}

				print("\n\n");
			}

			// Clear any lingering data from the fileline variable
			memset(fileline, 0, strlen(fileline));
		}

		// Close up the file on the SD card
		fclose(pFile);

		// Print the number of filelines encountered
		print("%u total lines", line_counter);
	}
 
	#else
    
	if(DEBUG && (debug_flags & DEBUG_ERROR))
	{
		print("ERROR - The proper inputs and outputs must be defined for SD card usage!");
	}
    
	#endif
    
	return 0;
}